@@ -17,19 +17,24 @@
 
 import os
 import re
+import sys
 from pathlib import Path
 from shutil import which
 from textwrap import dedent, indent, shorten
 
-from boltons.cacheutils import cachedproperty
+if sys.version_info >= (3, 8):
+    from functools import cached_property
+else:
+    from boltons.cacheutils import cachedproperty as cached_property
+
 from boltons.iterutils import flatten
 from boltons.strutils import strip_ansi
 from boltons.typeutils import classproperty
 from click_extra.colorize import theme
-from click_extra.logging import logger
 from click_extra.platform import CURRENT_OS_ID
-from click_extra.run import INDENT, PROMPT, run
+from click_extra.run import INDENT, format_cli, run_cmd
 
+from . import logger
 from .version import parse_version
 
 CLI_FORMATS = frozenset({"plain", "fragments", "xbar"})
@@ -54,11 +59,11 @@ class CLIError(Exception):
         return indent(
             dedent(
                 f"""
-            Return code: {self.code}
-            Output:
-            {indented_output}
-            Error:
-            {indented_error}"""
+                Return code: {self.code}
+                Output:
+                {indented_output}
+                Error:
+                {indented_error}"""
             ),
             INDENT,
         )
@@ -74,22 +79,40 @@ class PackageManager:
 
     """Base class from which all package manager definitions inherits."""
 
+    @classproperty
+    def id(cls):
+        """Return package manager's ID. Defaults based on class name.
+
+        This ID must be unique among all package manager definitions and lower-case as
+        they're used as feature flags for the :command:`mpm` CLI.
+        """
+        return cls.__name__.lower().replace("_", "-")
+
+    @classproperty
+    def name(cls):
+        """Return package manager's common name.
+
+        Defaults based on class name.
+        """
+        return cls.__name__
+
     platforms = frozenset()
     """List of platforms supported by the manager."""
 
     requirement = None
     """Minimal required version."""
 
-    version_cli_options = ("--version",)
-    """List of options to get the version from the package manager CLI."""
+    @classproperty
+    def cli_names(cls):
+        """List of CLI names the package manager is known as.
 
-    version_regex = r"(?P<version>\S+)"
-    """ Regular expression used to extract the version number from the result of the CLI
-    run with the options above. It doesn't matter if the regex returns unsanitized
-    and crappy string. The ``version()`` method will clean and normalized it.
+        The supported CLI names are ordered by priority. This is used for example to
+        help out the search of the right binary in the case of the python3/python2
+        transition.
 
-    By default match the first part that is space-separated.
-    """
+        Is derived by default from the manager's ID.
+        """
+        return (cls.id,)
 
     cli_search_path = ()
     """ List of additional path to help mpm hunt down the package manager CLI.
@@ -100,17 +123,41 @@ class PackageManager:
     works well on all platforms.
     """
 
-    global_args = ()
-    """Global list of options used for each call to the package manager CLI.
+    extra_env = None
+    """Additional environment variables to add to the current context.
+
+    Automaticcaly applied on each :py:func:`meta_package_manager.base.PackageManager.run_cli`
+    calls.
+    """
+
+    pre_cmds = ()
+    """Global list of pre-commands to add before before invoked CLI.
 
     Automaticcaly added to each :py:func:`meta_package_manager.base.PackageManager.run_cli`
     call.
 
-    Usually used to force silencing, low verbosity or no color output.
+    Used to prepend ``sudo`` or other CLIs.
     """
 
-    prepend_global_args = True
-    """Add the global args either at the begginning (``True``) or the end (``False``) of the CLI.
+    pre_args = ()
+    post_args = ()
+    """Global list of options used before and after the invoked package manager CLI.
+
+    Automaticcaly added to each :py:func:`meta_package_manager.base.PackageManager.run_cli`
+    call.
+
+    Essentially used to force silencing, low verbosity or no-color output.
+    """
+
+    version_cli_options = ("--version",)
+    """List of options to get the version from the package manager CLI."""
+
+    version_regex = r"(?P<version>\S+)"
+    """ Regular expression used to extract the version number from the result of the CLI
+    run with the options above. It doesn't matter if the regex returns unsanitized
+    and crappy string. The ``version()`` method will clean and normalized it.
+
+    By default match the first part that is space-separated.
     """
 
     stop_on_error = False
@@ -127,42 +174,16 @@ class PackageManager:
         # Log of all encountered CLI errors.
         self.cli_errors = []
 
-    @classproperty
-    def id(cls):
-        """Return package manager's ID. Defaults based on class name.
-
-        This ID must be unique among all package manager definitions and
-        lower-case as they're used as feature flags for the :command:`mpm` CLI.
-        """
-        return cls.__name__.lower().replace("_", "-")
-
-    @classproperty
-    def name(cls):
-        """Return package manager's common name. Defaults based on class name."""
-        return cls.__name__
-
-    @classproperty
-    def cli_names(cls):
-        """List of CLI names the package manager is known as.
-
-        The supported CLI names are ordered by priority. This is used for example to
-        help out the search of the right binary in the case of the python3/python2
-        transition.
-
-        Is derived by default from the manager's ID.
-        """
-        return (cls.id,)
-
     @classproperty
     def virtual(cls):
         """Should we expose the package manager to the user?
 
-        Virtual package manager are just skeleton classes used to factorize
-        code among managers of the same family.
+        Virtual package manager are just skeleton classes used to factorize code among
+        managers of the same family.
         """
         return cls.__name__ == "PackageManager" or not cls.cli_names
 
-    @cachedproperty
+    @cached_property
     def cli_path(self):
         """Fully qualified path to the package manager CLI.
 
@@ -182,7 +203,7 @@ class PackageManager:
             cli_path = which(name, mode=os.F_OK, path=env_path)
             if cli_path:
                 break
-            logger.debug(f"{name} CLI not found.")
+            logger.debug(f"{theme.invoked_command(name)} CLI not found.")
 
         if not cli_path:
             return
@@ -194,13 +215,15 @@ class PackageManager:
         if not cli_path.exists():
             raise FileNotFoundError(f"{cli_path}")
         elif not cli_path.is_file():
-            logger.warning(f"{cli_path} is not a file.")
+            logger.warning(f"{theme.invoked_command(cli_path)} is not a file.")
         else:
-            logger.debug(f"{name} CLI found at {cli_path}")
+            logger.debug(
+                f"{theme.invoked_command(name)} CLI found at {theme.invoked_command(cli_path)}"
+            )
 
         return cli_path
 
-    @cachedproperty
+    @cached_property
     def version(self):
         """Invoke the manager and extract its own reported version string.
 
@@ -209,36 +232,56 @@ class PackageManager:
         """
         if self.executable:
             # Invoke the manager.
-            output = self.run_cli(
-                self.version_cli_options, skip_globals=True, force_exec=True
-            )
+            try:
+                output = self.run_cli(
+                    self.version_cli_options,
+                    auto_pre_cmds=False,
+                    auto_pre_args=False,
+                    auto_post_args=False,
+                    force_exec=True,
+                )
+            # Catch false-positive CLIs.
+            except OSError as ex:
+                # In the environment on Windows, extension of available executables are
+                # ignored as they're plenty: .EXE, .CMD, .BAT, ...
+                # See: https://github.com/kdeldycke/meta-package-manager/issues/542
+                # Check for "OSError: [WinError 193] %1 is not a valid Win32
+                # application" error.
+                if getattr(ex, "winerror", None) == 193:
+                    logger.debug(
+                        f"{theme.invoked_command(self.cli_path)} is not a valid Windows application."
+                    )
+                    # Declare CLI as un-executable.
+                    self.executable = False
+                    return
+                # Unidentified error: re-raise.
+                raise
 
             # Extract the version with the regex.
-            if output:
-                version_string = (
-                    re.compile(self.version_regex, re.MULTILINE | re.VERBOSE)
-                    .search(output)
-                    .groupdict()
-                ).get("version")
-                logger.debug(f"Extracted version: {version_string}")
+            parts = re.compile(self.version_regex, re.MULTILINE | re.VERBOSE).search(
+                output
+            )
+            if parts:
+                version_string = parts.groupdict().get("version")
+                logger.debug(f"Extracted version: {version_string!r}")
                 return parse_version(version_string)
 
-    @cachedproperty
+    @cached_property
     def supported(self):
         """Is the package manager supported on that platform?"""
         return CURRENT_OS_ID in self.platforms
 
-    @cachedproperty
+    @cached_property
     def executable(self):
         """Is the package manager CLI can be executed by the current user?"""
         if not self.cli_path:
             return False
         if not os.access(self.cli_path, os.X_OK):
-            logger.debug(f"{self.cli_path} not executable.")
+            logger.debug(f"{self.cli_path} is not allowed to be executed.")
             return False
         return True
 
-    @cachedproperty
+    @cached_property
     def fresh(self):
         """Does the package manager match the version requirement?"""
         # Version is mandatory.
@@ -248,12 +291,12 @@ class PackageManager:
             if self.version < parse_version(self.requirement):
                 logger.debug(
                     f"{self.id} {self.version} is older than "
-                    "{self.requirement} version requirement."
+                    f"{self.requirement} version requirement."
                 )
                 return False
         return True
 
-    @cachedproperty
+    @cached_property
     def available(self):
         """Is the package manager available and ready-to-use on the system?
 
@@ -266,11 +309,20 @@ class PackageManager:
         """
         return bool(self.supported and self.cli_path and self.executable and self.fresh)
 
-    def run(self, *args):
+    @classmethod
+    def cleanup_cli(cls, *args):
+        """Flatten recusive iterables, remove `None`s, and cast to string each element.
+
+        Helps serialize Path and Version objects.
+        """
+        return tuple(map(str, filter(None.__ne__, flatten(args))))
+
+    def run(self, *args, extra_env=None):
         """Run a shell command, return the output and accumulate error messages.
 
-        args is allowed to be a nested structure of iterables, in which case it will
-        be recursively flatten, and each item within casted to strings.
+        ``args`` is allowed to be a nested structure of iterables, in which case it will
+        be recursively flatten, then ``None`` will be discarded, and finally each item
+        casted to strings.
 
         Running commands with that method:
           * adds logs at the appropriate level
@@ -283,27 +335,26 @@ class PackageManager:
             Move ``--dry-run`` option and this method to click-extra?
         """
         # Casting to string helps serialize Path and Version objects.
-        args = list(map(str, flatten(args)))
-        args_str = theme.invoked_command(" ".join(args))
-        cli_msg = f"{PROMPT}{args_str}"
+        args = self.cleanup_cli(args)
+        cli_msg = format_cli(args, extra_env)
 
         code = 0
-        output = None
-        error = None
+        output = ""
+        error = ""
 
         if self.dry_run:
             logger.warning(f"Dry-run: {cli_msg}")
         else:
             logger.debug(cli_msg)
-            code, output, error = run(*args)
+            code, output, error = run_cmd(
+                *args, extra_env=extra_env, print_output=False
+            )
 
         # Normalize messages.
         if error:
             error = dedent(strip_ansi(error).strip())
-            error = error if error else None
         if output:
             output = dedent(strip_ansi(output).strip())
-            output = output if output else None
 
         # Log <stdout> and <stderr> output.
         if output:
@@ -324,23 +375,115 @@ class PackageManager:
 
         return output
 
-    def run_cli(self, *args, skip_globals=False, force_exec=False):
-        """Shortcut utility to the ``run`` method above, that is explicitly using the
-        binary set by the ``cli_path`` property.
+    def build_cli(
+        self,
+        *args,
+        auto_pre_cmds=True,
+        auto_pre_args=True,
+        auto_post_args=True,
+        override_pre_cmds=False,
+        override_pre_args=False,
+        override_post_args=False,
+    ):
+        """Build the package manager CLI by combining the custom ``*args`` with the PM's
+        global parameters.
 
-        ``global_args`` are automaticcaly added before the provided args unless
-        ``prepend_global_args`` is ``False``. if ``skip_globals`` is ``True`` global arguments
-        are not added whatsoever to the list of provided ``args``.
+        Returns a tuple of strings.
 
-        Also offer the possibility to force the execution and completion of the command
-        regardless of ``--dry-run`` and ``--stop-on-error`` user options.
+        Helps the construction of CLI's repeating patterns and makes the code easier to read. Just pass the
+        specific ``*args`` and the full CLI string will be composed out of the globals,
+        following this schema:
+
+        .. code-block:: shell-session
+
+            $ <self.pre_cmds> <self.cli_path> <self.pre_args> <*args> <self.post_args>
+
+        * ``self.pre_cmds`` is added before the CLI path.
+
+        * ``self.cli_path`` is used as the main binary to execute.
+
+        * ``self.pre_args`` and ``self.post_args`` globals are added before and after
+          the provided ``*args``.
+
+        Each additional set of elements can be disabled with their respective flag:
+        * ``auto_pre_cmds=False``  to skip the automatic addition of ``self.pre_cmds``
+        * ``auto_pre_args=False``  to skip the automatic addition of ``self.pre_args``
+        * ``auto_post_args=False`` to skip the automatic addition of ``self.post_args``
+
+        Each global set of elements can be locally overriden with:
+        * ``override_pre_cmds=tuple()``
+        * ``override_pre_args=tuple()``
+        * ``override_post_args=tuple()``
         """
+        params = []
+
         # Prepare the full list of CLI arguments.
-        if not skip_globals:
-            if self.prepend_global_args:
-                args = list(self.global_args) + list(args)
-            else:
-                args = list(args) + list(self.global_args)
+        if override_pre_cmds:
+            assert isinstance(override_pre_cmds, tuple)
+        elif auto_pre_cmds:
+            params.extend(self.pre_cmds)
+
+        params.append(self.cli_path)
+
+        if override_pre_args:
+            assert isinstance(override_pre_args, tuple)
+        elif auto_pre_args:
+            params.extend(self.pre_args)
+
+        if args:
+            params.extend(args)
+
+        if override_post_args:
+            assert isinstance(override_post_args, tuple)
+        elif auto_post_args:
+            params.extend(self.post_args)
+
+        return self.cleanup_cli(params)
+
+    def run_cli(
+        self,
+        *args,
+        auto_extra_env=True,
+        auto_pre_cmds=True,
+        auto_pre_args=True,
+        auto_post_args=True,
+        override_extra_env=False,
+        override_pre_cmds=False,
+        override_pre_args=False,
+        override_post_args=False,
+        force_exec=False,
+    ):
+        """Build and run the package manager CLI by combining the custom ``*args`` with
+        the PM's global parameters.
+
+        After the CLI is built with the ``build_cli`` method, it is executed with the ``run`` method.
+
+        It reuse the ``build_cli`` method parameters and extend them with:
+
+        * ``self.extra_env`` for environment variables during execution.
+        * ``auto_extra_env=False``  to skip the automatic addition of ``self.extra_env``
+        * ``override_extra_env=dict()``
+
+        ``force_exec`` parameter ignores the ``--dry-run`` and ``--stop-on-error`` user options to force
+        the execution and completion of the command.
+        """
+        cli = self.build_cli(
+            *args,
+            auto_pre_cmds=auto_pre_cmds,
+            auto_pre_args=auto_pre_args,
+            auto_post_args=auto_post_args,
+            override_pre_cmds=override_pre_cmds,
+            override_pre_args=override_pre_args,
+            override_post_args=override_post_args,
+        )
+
+        # Prepare the full list of CLI arguments.
+        extra_env = None
+        if override_extra_env:
+            assert isinstance(override_extra_env, dict)
+            extra_env = override_extra_env
+        elif auto_extra_env:
+            extra_env = self.extra_env
 
         # Temporarily replace --dry-run and --stop-on-error user options with our own.
         if force_exec:
@@ -348,7 +491,7 @@ class PackageManager:
             self.dry_run, self.stop_on_error = False, False
 
         # Execute the command.
-        output = self.run(self.cli_path, args)
+        output = self.run(*cli, extra_env=extra_env)
 
         # Restore user options for --dry-run and --stop-on-error.
         if force_exec:
@@ -356,35 +499,29 @@ class PackageManager:
 
         return output
 
-    def sync(self):
-        """Refresh local manager metadata from remote repository."""
-        if self.sync.__func__.__qualname__ == PackageManager.sync.__qualname__:
-            logger.warning(f"{self.id} does not implement sync command.")
-            return
-        logger.info(f"Sync {self.id} package info...")
-
-    def cleanup(self):
-        """Remove left-overs and unused packages."""
-        if self.cleanup.__func__.__qualname__ == PackageManager.cleanup.__qualname__:
-            logger.warning(f"{self.id} does not implement cleanup command.")
-            return
-        logger.info(f"Cleanup {self.id}...")
-
     @property
     def installed(self):
         """List packages currently installed on the system.
 
-        Returns a dict indexed by package IDs. Each item is a dict with
-        package ID, name and version.
+        Returns a dict indexed by package IDs. Each item is a dict with package ID, name
+        and version.
+        """
+        raise NotImplementedError
+
+    @property
+    def outdated(self):
+        """List currently installed packages having a new version available.
+
+        Returns a dict indexed by package IDs. Each item is a dict with package ID,
+        name, current installed version and latest upgradeable version.
         """
         raise NotImplementedError
 
     def search(self, query, extended, exact):
         """Search packages whose ID contain exact or partial query.
 
-        Returns a dict indexed by package IDs. Each item is a dict with
-        package ID, name, version and a boolean indicating if the match is
-        exact or partial.
+        Returns a dict indexed by package IDs. Each item is a dict with package ID,
+        name, version and a boolean indicating if the match is exact or partial.
         """
         raise NotImplementedError
 
@@ -395,39 +532,30 @@ class PackageManager:
             return
         logger.info(f"Install {package_id} package from {self.id}...")
 
-    @property
-    def outdated(self):
-        """List currently installed packages having a new version available.
-
-        Returns a dict indexed by package IDs. Each item is a dict with
-        package ID, name, current installed version and latest upgradeable
-        version.
-        """
-        raise NotImplementedError
-
     def upgrade_cli(self, package_id=None):
         """Return a shell-compatible full-CLI to upgrade a package."""
         raise NotImplementedError
 
     def upgrade(self, package_id=None):
         """Perform the upgrade of the provided package to latest version."""
-        return self.run(self.upgrade_cli(package_id))
+        return self.run(self.upgrade_cli(package_id), extra_env=self.extra_env)
 
     def upgrade_all_cli(self):
         """Return a shell-compatible full-CLI to upgrade all packages.
 
-        Piggy-back on upgrade_cli() by default, which we'll call without a package_id parameter.
+        Piggy-back on upgrade_cli() by default, which we'll call without a package_id
+        parameter.
         """
         return self.upgrade_cli()
 
     def upgrade_all(self):
         """Perform a full upgrade of all outdated packages to latest versions.
 
-        If the manager doesn't implements a full upgrade one-liner, then
-        fall-back to calling single-package upgrade one by one.
+        If the manager doesn't implements a full upgrade one-liner, then fall-back to
+        calling single-package upgrade one by one.
         """
         try:
-            return self.run(self.upgrade_all_cli())
+            return self.run(self.upgrade_all_cli(), extra_env=self.extra_env)
         except NotImplementedError:
             logger.warning(f"{self.id} does not implement upgrade_all command.")
             logger.info(f"Call single-package upgrade CLI one by one.")
@@ -447,7 +575,7 @@ class PackageManager:
         * ``fragments`` returns a list of strings
         * ``xbar`` returns a CLI with parameters formatted into the xbar dialect
         """
-        assert isinstance(cmd, list)
+        assert isinstance(cmd, tuple)
         assert cli_format in CLI_FORMATS
         cmd = map(str, flatten(cmd))  # Serialize Path instances.
 
@@ -463,3 +591,17 @@ class PackageManager:
             param_id = "shell" if index == 0 else f"param{index}"
             xbar_params.append(f"{param_id}={param_value}")
         return " | ".join(xbar_params)
+
+    def sync(self):
+        """Refresh local manager metadata from remote repository."""
+        if self.sync.__func__.__qualname__ == PackageManager.sync.__qualname__:
+            logger.warning(f"{self.id} does not implement sync command.")
+            return
+        logger.info(f"Sync {self.id} package info...")
+
+    def cleanup(self):
+        """Remove left-overs and unused packages."""
+        if self.cleanup.__func__.__qualname__ == PackageManager.cleanup.__qualname__:
+            logger.warning(f"{self.id} does not implement cleanup command.")
+            return
+        logger.info(f"Cleanup {self.id}...")
