@@ -17,6 +17,7 @@
 
 import logging
 import re
+import sys
 from collections import namedtuple
 from datetime import datetime
 from functools import partial
@@ -24,19 +25,32 @@ from io import TextIOWrapper
 from operator import getitem
 from pathlib import Path
 
-import click
-import tomli
 import tomli_w
 from boltons.cacheutils import LRI, cached
 from boltons.strutils import complement_int_list, int_ranges_from_int_list
+from click_extra import (
+    STRING,
+    Choice,
+    File,
+    argument,
+    echo,
+    group,
+    option,
+    option_group,
+    pass_context,
+)
 from click_extra.colorize import KO, OK, theme
-from click_extra.commands import command, group
-from click_extra.logging import logger
 from click_extra.platform import os_label
 from click_extra.tabulate import table_format_option
-from cloup import option, option_group
 
-from . import __version__
+if sys.version_info >= (3, 11):
+    import tomllib
+else:
+    import tomli as tomllib
+
+from cloup import Section
+
+from . import __version__, logger, xbar
 from .base import CLI_FORMATS, CLIError, PackageManager
 from .output import (
     SORTABLE_FIELDS,
@@ -45,11 +59,16 @@ from .output import (
     print_stats,
     print_table,
 )
-from .pool import ALL_MANAGER_IDS, select_managers
+from .pool import pool
 from .version import TokenizedString
 
-XKCD_MANAGER_ORDER = ("pip", "brew", "npm", "apt")
-assert set(ALL_MANAGER_IDS).issuperset(XKCD_MANAGER_ORDER)
+# Sub-command sections.
+EXPLORE = Section("Explore commands")
+MAINTENANCE = Section("Maintainance commands")
+SNAPSHOTS = Section("Package snapshots commands")
+
+
+XKCD_MANAGER_ORDER = ("pip", "brew", "npm", "dnf", "apt")
 
 
 def add_manager_to_selection(ctx, param, selected):
@@ -57,12 +76,16 @@ def add_manager_to_selection(ctx, param, selected):
     if selected:
         if ctx.obj is None:
             ctx.obj = {"single_manager_selector": []}
-        ctx.obj["single_manager_selector"].append(param.name)
+        # Parameter's name is transformed into a Python identifier on instanciation.
+        # Reverse the process to get our value.
+        # Example: "--apt-mint" => "apt_mint" => "apt-mint"
+        manager_id = param.name.replace("_", "-")
+        ctx.obj["single_manager_selector"].append(manager_id)
 
 
 def single_manager_selectors():
     """Dynamiccaly creates a dedicated flag selector for each manager."""
-    for manager_id in ALL_MANAGER_IDS:
+    for manager_id in pool.all_manager_ids:
         yield option(
             f"--{manager_id}",
             is_flag=True,
@@ -73,13 +96,30 @@ def single_manager_selectors():
         )
 
 
+def xbar_plugin_path(ctx, param, value):
+    """Print the location of the xbar plugin.
+
+    Replaces the fully-qualified home directory by its shorthand if applicable.
+    """
+    if value:
+        xbar_path = Path(xbar.__file__).expanduser().resolve()
+        home_dir = Path.home()
+        if xbar_path.is_relative_to(home_dir):
+            home_shorthand = Path("~")
+            shorten_xbar_path = home_shorthand / xbar_path.relative_to(home_dir)
+            assert shorten_xbar_path.expanduser().resolve() == xbar_path
+            xbar_path = shorten_xbar_path
+        echo(xbar_path)
+        ctx.exit()
+
+
 @group(version=__version__)
 @option_group(
     "Package manager selection options",
     option(
         "-m",
         "--manager",
-        type=click.Choice(ALL_MANAGER_IDS, case_sensitive=False),
+        type=Choice(pool.all_manager_ids, case_sensitive=False),
         multiple=True,
         help="Restrict sub-command to a subset of package managers. Repeat to "
         "select multiple managers. The order in which options are provided defines the "
@@ -88,7 +128,7 @@ def single_manager_selectors():
     option(
         "-e",
         "--exclude",
-        type=click.Choice(ALL_MANAGER_IDS, case_sensitive=False),
+        type=Choice(pool.all_manager_ids, case_sensitive=False),
         multiple=True,
         help="Exclude a package manager. Repeat to exclude multiple managers.",
     ),
@@ -137,7 +177,7 @@ def single_manager_selectors():
     table_format_option(
         "-o",
         "--output-format",
-        type=click.Choice(
+        type=Choice(
             sorted(TabularOutputFormatter._output_formats), case_sensitive=False
         ),
         help="Rendering mode of the output.",
@@ -145,7 +185,7 @@ def single_manager_selectors():
     option(
         "-s",
         "--sort-by",
-        type=click.Choice(sorted(SORTABLE_FIELDS), case_sensitive=False),
+        type=Choice(sorted(SORTABLE_FIELDS), case_sensitive=False),
         default="manager_id",
         help="Sort results.",
     ),
@@ -156,7 +196,16 @@ def single_manager_selectors():
         help="Print per-manager package statistics.",
     ),
 )
-@click.pass_context
+@option(
+    "--xbar-plugin-path",
+    is_flag=True,
+    default=False,
+    expose_value=False,
+    is_eager=True,
+    callback=xbar_plugin_path,
+    help="Print location of the mpm xbar plugin.",
+)
+@pass_context
 def mpm(
     ctx,
     manager,
@@ -175,8 +224,14 @@ def mpm(
     if ctx.obj:
         manager = list(manager) + ctx.obj.get("single_manager_selector", [])
 
+    # Silence all log message for JSON rendering unless in debug mode.
+    level = logger.level
+    level_name = logging._levelToName.get(level, level)
+    if ctx.find_root().table_formatter.format_name == "json" and level_name != "DEBUG":
+        logger.setLevel(logging.CRITICAL * 2)
+
     # Select the subset of manager to target, and apply manager-level options.
-    selected_managers = select_managers(
+    selected_managers = pool.select_managers(
         keep=manager if not xkcd else XKCD_MANAGER_ORDER,
         drop=exclude,
         drop_unsupported=not all_managers,
@@ -190,12 +245,6 @@ def mpm(
         dry_run=dry_run,
     )
 
-    # Silence all log message for JSON rendering unless in debug mode.
-    level = logger.level
-    level_name = logging._levelToName.get(level, level)
-    if ctx.find_root().table_formatter.format_name == "json" and level_name != "DEBUG":
-        logger.setLevel(logging.CRITICAL * 2)
-
     # Load up current and new global options to the context for subcommand consumption.
     ctx.obj = namedtuple(
         "GlobalOptions",
@@ -212,8 +261,10 @@ def mpm(
     )()
 
 
-@command(short_help="List supported package managers and their location.")
-@click.pass_context
+@mpm.command(
+    short_help="List supported package managers and their location.", section=EXPLORE
+)
+@pass_context
 def managers(ctx):
     """List all supported package managers and their presence on the system."""
     # Machine-friendly data rendering.
@@ -248,7 +299,7 @@ def managers(ctx):
         os_infos = OK if manager.supported else KO
         if not manager.supported:
             os_infos += " {} only".format(
-                ", ".join(sorted((os_label(os_id) for os_id in manager.platforms)))
+                ", ".join(sorted(os_label(os_id) for os_id in manager.platforms))
             )
 
         # Build up the CLI path column content.
@@ -293,24 +344,8 @@ def managers(ctx):
     )
 
 
-@command(short_help="Sync local package info.")
-@click.pass_context
-def sync(ctx):
-    """Sync local package metadata and info from external sources."""
-    for manager in ctx.obj.selected_managers:
-        manager.sync()
-
-
-@command(short_help="Cleanup local data.")
-@click.pass_context
-def cleanup(ctx):
-    """Cleanup local data and temporary artifacts."""
-    for manager in ctx.obj.selected_managers:
-        manager.cleanup()
-
-
-@command(short_help="List installed packages.")
-@click.pass_context
+@mpm.command(short_help="List installed packages.", section=EXPLORE)
+@pass_context
 def installed(ctx):
     """List all packages installed on the system from all managers."""
     # Build-up a global dict of installed packages per manager.
@@ -362,7 +397,103 @@ def installed(ctx):
         print_stats(installed_data)
 
 
-@command(short_help="Search packages.")
+@mpm.command(short_help="List outdated packages.", section=EXPLORE)
+@option(
+    "-c",
+    "--cli-format",
+    type=Choice(sorted(CLI_FORMATS), case_sensitive=False),
+    default="plain",
+    help="Format of CLI fields in JSON output.",
+)
+@pass_context
+def outdated(ctx, cli_format):
+    """List available package upgrades and their versions for each manager."""
+    render_cli = partial(PackageManager.render_cli, cli_format=cli_format)
+
+    # Build-up a global list of outdated packages per manager.
+    outdated_data = {}
+
+    for manager in ctx.obj.selected_managers:
+
+        try:
+            packages = tuple(map(dict, manager.outdated.values()))
+        except NotImplementedError:
+            logger.warning(f"{manager.id} does not implement outdated command.")
+            continue
+
+        for info in packages:
+            info.update({"upgrade_cli": render_cli(manager.upgrade_cli(info["id"]))})
+
+        outdated_data[manager.id] = {
+            "id": manager.id,
+            "name": manager.name,
+            "packages": packages,
+        }
+
+        # Do not include the full-upgrade CLI if we did not detect any outdated
+        # package.
+        if packages:
+            try:
+                upgrade_all_cli = manager.upgrade_all_cli()
+            except NotImplementedError:
+                # Fallback on mpm itself which is capable of simulating a full
+                # upgrade.
+                upgrade_all_cli = ("mpm", "--manager", manager.id, "upgrade")
+            outdated_data[manager.id]["upgrade_all_cli"] = render_cli(upgrade_all_cli)
+
+        # Serialize errors at the last minute to gather all we encountered.
+        outdated_data[manager.id]["errors"] = list(
+            {expt.error for expt in manager.cli_errors}
+        )
+
+    # Machine-friendly data rendering.
+    if ctx.find_root().table_formatter.format_name == "json":
+        print_json(outdated_data)
+        return
+
+    # Human-friendly content rendering.
+    table = []
+    for manager_id, outdated_pkg in outdated_data.items():
+        table += [
+            (
+                info["name"],
+                info["id"],
+                manager_id,
+                info["installed_version"] if info["installed_version"] else "?",
+                info["latest_version"],
+            )
+            for info in outdated_pkg["packages"]
+        ]
+
+    # Sort and print table.
+    print_table(
+        (
+            ("Package name", "package_name"),
+            ("ID", "package_id"),
+            ("Manager", "manager_id"),
+            ("Installed version", "version"),
+            ("Latest version", None),
+        ),
+        table,
+        ctx.obj.sort_by,
+    )
+
+    if ctx.obj.stats:
+        print_stats(outdated_data)
+
+
+# TODO: make it a --search-strategy=[exact, fuzzy, extended]
+# Add details helps => exact: is case-sensitive, and keep all non-alnum chars
+# fuzzy: query is case-insensitive, stripped-out of non-alnum chars and
+# tokenized (no order sensitive)
+# extended, same as fuzzy, but do not limit search to package name and ID.
+# extended to description and other metadata depending on manager support.
+# Modes:
+#  1. strict (--exact)
+#  2. substring (regex, no case, no split)
+#  3. fuzzy (token-based)
+#  4. extended (+ metadata)
+@mpm.command(short_help="Search packages.", section=EXPLORE)
 @option(
     "--extended/--package-name",
     default=False,
@@ -374,8 +505,8 @@ def installed(ctx):
     default=False,
     help="Only returns exact matches, or enable fuzzy search in substrings.",
 )
-@click.argument("query", type=click.STRING, required=True)
-@click.pass_context
+@argument("query", type=STRING, required=True)
+@pass_context
 def search(ctx, extended, exact, query):
     """Search packages from all managers."""
     # Build-up a global list of package matches per manager.
@@ -471,9 +602,10 @@ def search(ctx, extended, exact, query):
         print_stats(matches)
 
 
-@command(short_help="Install a package.")
-@click.argument("package_id", type=click.STRING, required=True)
-@click.pass_context
+@mpm.command(short_help="Install a package.", section=MAINTENANCE)
+@argument("package_id", type=STRING, required=True)
+# TODO: add a --force/--reinstall flag
+@pass_context
 def install(ctx, package_id):
     """Install the provided package using one of the provided package manager."""
     # Cast generator to tuple because of reuse.
@@ -516,97 +648,12 @@ def install(ctx, package_id):
         # Restore default value.
         manager.stop_on_error = default_value
 
-        click.echo(output)
+        echo(output)
         return
 
 
-@command(short_help="List outdated packages.")
-@option(
-    "-c",
-    "--cli-format",
-    type=click.Choice(sorted(CLI_FORMATS), case_sensitive=False),
-    default="plain",
-    help="Format of CLI fields in JSON output.",
-)
-@click.pass_context
-def outdated(ctx, cli_format):
-    """List available package upgrades and their versions for each manager."""
-    render_cli = partial(PackageManager.render_cli, cli_format=cli_format)
-
-    # Build-up a global list of outdated packages per manager.
-    outdated_data = {}
-
-    for manager in ctx.obj.selected_managers:
-
-        try:
-            packages = tuple(map(dict, manager.outdated.values()))
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement outdated command.")
-            continue
-
-        for info in packages:
-            info.update({"upgrade_cli": render_cli(manager.upgrade_cli(info["id"]))})
-
-        outdated_data[manager.id] = {
-            "id": manager.id,
-            "name": manager.name,
-            "packages": packages,
-        }
-
-        # Do not include the full-upgrade CLI if we did not detect any outdated
-        # package.
-        if packages:
-            try:
-                upgrade_all_cli = manager.upgrade_all_cli()
-            except NotImplementedError:
-                # Fallback on mpm itself which is capable of simulating a full
-                # upgrade.
-                upgrade_all_cli = ["mpm", "--manager", manager.id, "upgrade"]
-            outdated_data[manager.id]["upgrade_all_cli"] = render_cli(upgrade_all_cli)
-
-        # Serialize errors at the last minute to gather all we encountered.
-        outdated_data[manager.id]["errors"] = list(
-            {expt.error for expt in manager.cli_errors}
-        )
-
-    # Machine-friendly data rendering.
-    if ctx.find_root().table_formatter.format_name == "json":
-        print_json(outdated_data)
-        return
-
-    # Human-friendly content rendering.
-    table = []
-    for manager_id, outdated_pkg in outdated_data.items():
-        table += [
-            (
-                info["name"],
-                info["id"],
-                manager_id,
-                info["installed_version"] if info["installed_version"] else "?",
-                info["latest_version"],
-            )
-            for info in outdated_pkg["packages"]
-        ]
-
-    # Sort and print table.
-    print_table(
-        (
-            ("Package name", "package_name"),
-            ("ID", "package_id"),
-            ("Manager", "manager_id"),
-            ("Installed version", "version"),
-            ("Latest version", None),
-        ),
-        table,
-        ctx.obj.sort_by,
-    )
-
-    if ctx.obj.stats:
-        print_stats(outdated_data)
-
-
-@command(short_help="Upgrade all packages.")
-@click.pass_context
+@mpm.command(short_help="Upgrade all packages.", section=MAINTENANCE)
+@pass_context
 def upgrade(ctx):
     """Perform a full package upgrade on all available managers."""
     for manager in ctx.obj.selected_managers:
@@ -621,9 +668,25 @@ def upgrade(ctx):
             logger.info(output)
 
 
-@command(short_help="Save installed packages to a TOML file.")
-@click.argument("toml_output", type=click.File("w"), default="-")
-@click.pass_context
+@mpm.command(short_help="Sync local package info.", section=MAINTENANCE)
+@pass_context
+def sync(ctx):
+    """Sync local package metadata and info from external sources."""
+    for manager in ctx.obj.selected_managers:
+        manager.sync()
+
+
+@mpm.command(short_help="Cleanup local data.", section=MAINTENANCE)
+@pass_context
+def cleanup(ctx):
+    """Cleanup local data and temporary artifacts."""
+    for manager in ctx.obj.selected_managers:
+        manager.cleanup()
+
+
+@mpm.command(short_help="Save installed packages to a TOML file.", section=SNAPSHOTS)
+@argument("toml_output", type=File("w"), default="-")
+@pass_context
 def backup(ctx, toml_output):
     """Dump the list of installed packages to a TOML file.
 
@@ -648,7 +711,7 @@ def backup(ctx, toml_output):
 
     # Leave some metadata as comment.
     doc = f"# Generated by mpm v{__version__}.\n"
-    doc += "# Timestamp: {}.\n".format(datetime.now().isoformat())
+    doc += f"# Timestamp: {datetime.now().isoformat()}.\n"
 
     installed_data = {}
 
@@ -664,7 +727,7 @@ def backup(ctx, toml_output):
         }
 
         pkg_data = dict(
-            sorted(((p["id"], str(p["installed_version"])) for p in installed_packages))
+            sorted((p["id"], str(p["installed_version"])) for p in installed_packages)
         )
 
         if pkg_data:
@@ -676,12 +739,14 @@ def backup(ctx, toml_output):
         print_stats(installed_data)
 
 
-@command(short_help="Install packages in batch as specified by TOML files.")
-@click.argument("toml_files", type=click.File("r"), required=True, nargs=-1)
-@click.pass_context
+@mpm.command(
+    short_help="Install packages in batch as specified by TOML files.",
+    section=SNAPSHOTS,
+)
+@argument("toml_files", type=File("r"), required=True, nargs=-1)
+@pass_context
 def restore(ctx, toml_files):
-    """Read TOML files then install or upgrade each package referenced in
-    them.
+    """Read TOML files then install or upgrade each package referenced in them.
 
     Version specified in the TOML file is ignored in the current implementation.
     """
@@ -691,11 +756,11 @@ def restore(ctx, toml_files):
         toml_filepath = toml_input.name if is_stdin else Path(toml_input.name).resolve()
         logger.info(f"Load package list from {toml_filepath}")
 
-        doc = tomli.loads(toml_input.read())
+        doc = tomllib.loads(toml_input.read())
 
         # List unrecognized sections.
         ignored_sections = [
-            f"[{section}]" for section in doc if section not in ALL_MANAGER_IDS
+            f"[{section}]" for section in doc if section not in pool.all_manager_ids
         ]
         if ignored_sections:
             plural = "s" if len(ignored_sections) > 1 else ""
@@ -709,28 +774,4 @@ def restore(ctx, toml_files):
             logger.info(f"Restore {manager.id} packages...")
             for package_id, version in doc[manager.id].items():
                 output = manager.install(package_id)
-                click.echo(output)
-
-
-# Group sub-command in sections.
-mpm.section(
-    "Explore commands",
-    managers,
-    installed,
-    outdated,
-    search,
-)
-
-mpm.section(
-    "Maintainance commands",
-    install,
-    upgrade,
-    sync,
-    cleanup,
-)
-
-mpm.section(
-    "Package snapshots commands",
-    backup,
-    restore,
-)
+                echo(output)
