@@ -43,23 +43,6 @@ class APT(PackageManager):
         apt 1.2.15 (amd64)
     """
 
-    def sync(self):
-        """
-        .. code-block:: shell-session
-
-            ► apt update --quiet
-            Hit:1 http://archive.ubuntu.com xenial InRelease
-            Get:2 http://archive.ubuntu.com xenial-updates InRelease [102 kB]
-            Get:3 http://archive.ubuntu.com xenial-security InRelease [102 kB]
-            Get:4 http://archive.ubuntu.com xenial/main Translation-en [568 kB]
-            Fetched 6,868 kB in 2s (2,680 kB/s)
-            Reading package lists...
-            Building dependency tree...
-            Reading state information...
-        """
-        super().sync()
-        self.run_cli("update", "--quiet")
-
     @property
     def installed(self):
         """Fetch installed packages from ``apt list`` output.
@@ -99,26 +82,56 @@ class APT(PackageManager):
 
         output = self.run_cli("list", "--installed", "--quiet")
 
-        if output:
-            regexp = re.compile(r"(\S+)\/\S+ (\S+) .*")
-            for package in output.splitlines():
-                match = regexp.match(package)
-                if match:
-                    package_id, installed_version = match.groups()
-                    installed[package_id] = {
-                        "id": package_id,
-                        "name": package_id,
-                        "installed_version": parse_version(installed_version),
-                    }
+        regexp = re.compile(r"(\S+)\/\S+ (\S+) .*")
+        for package in output.splitlines():
+            match = regexp.match(package)
+            if match:
+                package_id, installed_version = match.groups()
+                installed[package_id] = {
+                    "id": package_id,
+                    "name": package_id,
+                    "installed_version": parse_version(installed_version),
+                }
 
         return installed
 
+    @property
+    def outdated(self):
+        """Fetch outdated packages from ``apt list`` output.
+
+        Raw CLI output samples:
+
+        .. code-block:: shell-session
+
+            ► apt list --upgradable --quiet
+            Listing...
+            apt/xenial-updates 1.2.19 amd64 [upgradable from: 1.2.15ubuntu0.2]
+            nano/xenial-updates 2.5.3-2ubuntu2 amd64 [upgradable from: 2.5.3-2]
+        """
+        outdated = {}
+
+        output = self.run_cli("list", "--upgradable", "--quiet")
+
+        regexp = re.compile(r"(\S+)\/\S+ (\S+).*\[upgradable from: (\S+)\]")
+        for package in output.splitlines():
+            match = regexp.match(package)
+            if match:
+                package_id, latest_version, installed_version = match.groups()
+                outdated[package_id] = {
+                    "id": package_id,
+                    "name": package_id,
+                    "latest_version": parse_version(latest_version),
+                    "installed_version": parse_version(installed_version),
+                }
+
+        return outdated
+
     def search(self, query, extended, exact):
         """Fetch matching packages from ``apt search`` output.
 
         .. code-block:: shell-session
 
-            ► apt search abc --names-only --quiet
+            ► apt search abc --quiet --names-only
             Sorting...
             Full Text Search...
             abcde/xenial 2.7.1-1 all
@@ -138,7 +151,7 @@ class APT(PackageManager):
 
         .. code-block:: shell-session
 
-            ► apt search "^sed$" --names-only --quiet
+            ► apt search "^sed$" --quiet --names-only
             Sorting...
             Full Text Search...
             sed/xenial 2.1.9-3 all
@@ -146,7 +159,7 @@ class APT(PackageManager):
 
         .. code-block:: shell-session
 
-            ► apt search abc --full --quiet
+            ► apt search abc --quiet --full
             Sorting...
             Full Text Search...
             abcde/xenial 2.7.1-1 all
@@ -168,7 +181,7 @@ class APT(PackageManager):
 
         search_arg = "--names-only"
         if exact:
-            # Realy on apt regexp support to speed-up exact match.
+            # Rely on apt regexp support to speed-up exact match.
             query = f"^{query}$"
         # Extended search are always non-exact.
         elif extended:
@@ -178,27 +191,26 @@ class APT(PackageManager):
 
         output = self.run_cli("search", query, "--quiet", search_arg)
 
-        if output:
-            regexp = re.compile(
-                r"""
-                ^(?P<package_id>\S+)  # A string with a char at least.
-                /.+\                  # A slash, any string, then a space.
-                (?P<version>.+)       # Any string.
-                \                     # A space.
-                (?:.+)\n              # Any string ending the line.
-                (?P<description>      # Start of the multi-line desc group.
-                    (?:\ \ .+\n)+     # Lines of strings prefixed by 2 spaces.
-                )
-                """,
-                re.MULTILINE | re.VERBOSE,
+        regexp = re.compile(
+            r"""
+            ^(?P<package_id>\S+)  # A string with a char at least.
+            /.+\                  # A slash, any string, then a space.
+            (?P<version>.+)       # Any string.
+            \                     # A space.
+            (?:.+)\n              # Any string ending the line.
+            (?P<description>      # Start of the multi-line desc group.
+                (?:\ \ .+\n)+     # Lines of strings prefixed by 2 spaces.
             )
+            """,
+            re.MULTILINE | re.VERBOSE,
+        )
 
-            for package_id, version, description in regexp.findall(output):
-                matches[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "latest_version": parse_version(version),
-                }
+        for package_id, version, description in regexp.findall(output):
+            matches[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "latest_version": parse_version(version),
+            }
 
         return matches
 
@@ -207,61 +219,42 @@ class APT(PackageManager):
 
         .. code-block:: shell-session
 
-            ► apt install --quiet bat
-
+            ► apt install bat
         """
         super().install(package_id)
         return self.run_cli("install", package_id)
 
-    @property
-    def outdated(self):
-        """Fetch outdated packages from ``apt list`` output.
-
-        Raw CLI output samples:
-
-        .. code-block:: shell-session
-
-            ► apt list --upgradable --quiet
-            Listing...
-            apt/xenial-updates 1.2.19 amd64 [upgradable from: 1.2.15ubuntu0.2]
-            nano/xenial-updates 2.5.3-2ubuntu2 amd64 [upgradable from: 2.5.3-2]
-        """
-        outdated = {}
-
-        output = self.run_cli("list", "--upgradable", "--quiet")
-
-        if output:
-            regexp = re.compile(r"(\S+)\/\S+ (\S+).*\[upgradable from: (\S+)\]")
-            for package in output.splitlines():
-                match = regexp.match(package)
-                if match:
-                    package_id, latest_version, installed_version = match.groups()
-                    outdated[package_id] = {
-                        "id": package_id,
-                        "name": package_id,
-                        "latest_version": parse_version(latest_version),
-                        "installed_version": parse_version(installed_version),
-                    }
-
-        return outdated
-
     def upgrade_cli(self, package_id=None):
-        cmd = [self.cli_path, "update"]
-        if package_id:
-            cmd.append(package_id)
-        return cmd
+        """.. code-block:: shell-session.
+
+        ► apt update bat
+        """
+        return self.build_cli("update", package_id)
+
+    def sync(self):
+        """.. code-block:: shell-session.
+
+        ► apt update --quiet Hit:1 http://archive.ubuntu.com xenial InRelease Get:2
+        http://archive.ubuntu.com xenial-updates InRelease [102 kB] Get:3
+        http://archive.ubuntu.com xenial-security InRelease [102 kB] Get:4
+        http://archive.ubuntu.com xenial/main Translation-en [568 kB] Fetched 6,868 kB
+        in 2s (2,680 kB/s) Reading package lists... Building dependency tree... Reading
+        state information...
+        """
+        super().sync()
+        self.run_cli("update", "--quiet")
 
     def cleanup(self):
         """Runs:
 
         .. code-block:: shell-session
 
-            ► sudo apt-get -y autoremove
+            ► sudo apt -y autoremove
+            ► sudo apt clean
         """
         super().cleanup()
-        # Cannot use self.run_cli() because of sudo.
-        self.run("sudo", self.cli_path, "-y", "autoremove")
-        self.run("sudo", self.cli_path, "clean")
+        for commands in (("-y", "autoremove"), ("clean",)):
+            self.run_cli(*commands, override_pre_cmds=("sudo",))
 
 
 class APT_Mint(APT):
@@ -282,3 +275,49 @@ class APT_Mint(APT):
         ► apt version apt
         1.6.11
     """
+
+    def search(self, query, extended, exact):
+        """.. code-block:: shell-session.
+
+            ► /usr/local/bin/apt search --quiet sed
+            v   librust-slog-2.5+erased-serde-dev  -
+            p   python3-blessed                    - Practical wrapper
+            i   sed                                - GNU stream editor
+            p   sed:i386                           - GNU stream editor
+
+        .. code-block:: shell-session
+
+            ► /usr/local/bin/apt search --quiet ^sed$
+            i   sed              - GNU stream editor
+            p   sed:i386         - GNU stream editor
+        """
+        matches = {}
+
+        if exact:
+            # Rely on apt regexp support to speed-up exact match.
+            query = f"^{query}$"
+
+        output = self.run_cli("search", "--quiet", query)
+
+        regexp = re.compile(
+            r"""
+            \S                       # One non-space character.
+            \s+                      # One space or more.
+            (?P<package_id>[^\s:]+)  # Any non-space until whitespace or semi-colon.
+            (?:\:\S+)?               # Optional arch suffix after package and semi-colon.
+            \s+                      # One space or more.
+            -                        # A dash.
+            \ ?                      # An optional space.
+            (?P<description>\S+)?    # Optional non-space string.
+            """,
+            re.VERBOSE,
+        )
+
+        for package_id, description in regexp.findall(output):
+            matches[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "latest_version": None,
+            }
+
+        return matches
