@@ -15,9 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+import json
 import re
 
-import simplejson as json
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
 from ..base import PackageManager
@@ -27,10 +27,11 @@ from ..version import parse_version
 class Composer(PackageManager):
 
     name = "PHP's Composer"
-    global_args = ("global",)
     platforms = frozenset({LINUX, MACOS, WINDOWS})
     requirement = "1.4.0"
 
+    pre_args = ("global",)
+
     version_regex = r"Composer\s+version\s+(?P<version>\S+)"
     """
     .. code-block:: shell-session
@@ -89,6 +90,51 @@ class Composer(PackageManager):
 
         return installed
 
+    @property
+    def outdated(self):
+        """Fetch outdated packages from ``composer global outdated`` output.
+
+        Raw CLI output samples:
+
+        .. code-block:: shell-session
+
+        ► composer global outdated --format=json
+        {
+            "installed": [
+                {
+                    "name": "illuminate/contracts",
+                    "version": "v5.7.2",
+                    "latest": "v5.7.3",
+                    "latest-status": "semver-safe-update",
+                    "description": "The Illuminate Contracts package."
+                },
+                {
+                    "name": "illuminate/support",
+                    "version": "v5.7.2",
+                    "latest": "v5.7.3",
+                    "latest-status": "semver-safe-update",
+                    "description": "The Illuminate Support package."
+                }
+            ]
+        }
+        """
+        outdated = {}
+
+        output = self.run_cli("outdated", "--format=json")
+
+        if output:
+            package_list = json.loads(output)
+            for package in package_list["installed"]:
+                package_id = package["name"]
+                outdated[package_id] = {
+                    "id": package_id,
+                    "name": package_id,
+                    "installed_version": parse_version(package["version"]),
+                    "latest_version": parse_version(package["latest"]),
+                }
+
+        return outdated
+
     def search(self, query, extended, exact):
         """Fetch matching packages from ``composer search`` output.
 
@@ -136,28 +182,25 @@ class Composer(PackageManager):
 
         output = self.run_cli("search", search_args, query)
 
-        if output:
+        regexp = re.compile(
+            r"""
+            ^(?P<package_id>\S+\/\S+)
+            (?P<description> .*)?
+            """,
+            re.MULTILINE | re.VERBOSE,
+        )
 
-            regexp = re.compile(
-                r"""
-                ^(?P<package_id>\S+\/\S+)
-                (?P<description> .*)?
-                """,
-                re.MULTILINE | re.VERBOSE,
-            )
+        for package_id, description in regexp.findall(output):
+            # Filters out fuzzy matches, only keep stricly matching
+            # packages.
+            if exact and query != package_id:
+                continue
 
-            for package_id, description in regexp.findall(output):
-
-                # Filters out fuzzy matches, only keep stricly matching
-                # packages.
-                if exact and query != package_id:
-                    continue
-
-                matches[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "latest_version": None,
-                }
+            matches[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "latest_version": None,
+            }
 
         return matches
 
@@ -167,61 +210,16 @@ class Composer(PackageManager):
         .. code-block:: shell-session
 
             ► composer global install illuminate/contracts
-
         """
         super().install(package_id)
         return self.run_cli("install", package_id)
 
-    @property
-    def outdated(self):
-        """Fetch outdated packages from ``composer global outdated`` output.
-
-        Raw CLI output samples:
-
-        .. code-block:: shell-session
-
-        ► composer global outdated --format=json
-        {
-            "installed": [
-                {
-                    "name": "illuminate/contracts",
-                    "version": "v5.7.2",
-                    "latest": "v5.7.3",
-                    "latest-status": "semver-safe-update",
-                    "description": "The Illuminate Contracts package."
-                },
-                {
-                    "name": "illuminate/support",
-                    "version": "v5.7.2",
-                    "latest": "v5.7.3",
-                    "latest-status": "semver-safe-update",
-                    "description": "The Illuminate Support package."
-                }
-            ]
-        }
-        """
-        outdated = {}
-
-        output = self.run_cli("outdated", "--format=json")
-
-        if output:
-            package_list = json.loads(output)
-            for package in package_list["installed"]:
-                package_id = package["name"]
-                outdated[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "installed_version": parse_version(package["version"]),
-                    "latest_version": parse_version(package["latest"]),
-                }
-
-        return outdated
-
     def upgrade_cli(self, package_id=None):
-        cmd = [self.cli_path, self.global_args, "update"]
-        if package_id:
-            cmd.append(package_id)
-        return cmd
+        """.. code-block:: shell-session.
+
+        ► composer global update illuminate/contracts
+        """
+        return self.build_cli("update", package_id)
 
     def cleanup(self):
         """Runs:
