@@ -17,19 +17,19 @@
 
 import re
 
-from click_extra.logging import logger
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
+from .. import logger
 from ..base import PackageManager
 from ..version import parse_version
 
 
 class Gem(PackageManager):
 
-    platforms = frozenset({LINUX, MACOS, WINDOWS})
-
     name = "Ruby Gems"
 
+    platforms = frozenset({LINUX, MACOS, WINDOWS})
+
     # Default to the version shipped with the latest maintained macOS version,
     # i.e. macOS 10.13 High Sierra, which is bundled with gem 2.5.2.
     requirement = "2.5.0"
@@ -44,8 +44,7 @@ class Gem(PackageManager):
     # Help mpm a little bit in its search for the `gem` binary.
     cli_search_path = ("/usr/local/opt/ruby/bin/gem", "/usr/local/opt/ruby/bin")
 
-    global_args = ("--quiet",)  # Silence command progress meter
-    prepend_global_args = False
+    post_args = ("--quiet",)  # Silence command progress meter
 
     @property
     def installed(self):
@@ -77,26 +76,59 @@ class Gem(PackageManager):
 
         output = self.run_cli("list")
 
-        if output:
-            regexp = re.compile(r"(\S+) \((.+)\)")
-            for package in output.splitlines():
-                match = regexp.match(package)
-                if match:
-                    package_id, versions = match.groups()
-                    # Guess latest installed version.
-                    version = max(
-                        parse_version(v)
-                        for v in re.compile(r",|default:| ").split(versions)
-                        if v
-                    )
-                    installed[package_id] = {
-                        "id": package_id,
-                        "name": package_id,
-                        "installed_version": version,
-                    }
+        regexp = re.compile(r"(\S+) \((.+)\)")
+        for package in output.splitlines():
+            match = regexp.match(package)
+            if match:
+                package_id, versions = match.groups()
+                # Guess latest installed version.
+                version = max(
+                    parse_version(v)
+                    for v in re.compile(r",|default:| ").split(versions)
+                    if v
+                )
+                installed[package_id] = {
+                    "id": package_id,
+                    "name": package_id,
+                    "installed_version": version,
+                }
 
         return installed
 
+    @property
+    def outdated(self):
+        """Fetch outdated packages from ``gem outdated`` output.
+
+        Raw CLI output samples:
+
+        .. code-block:: shell-session
+
+            ► gem outdated --quiet
+            did_you_mean (1.0.0 < 1.0.2)
+            io-console (0.4.5 < 0.4.6)
+            json (1.8.3 < 2.0.1)
+            minitest (5.8.3 < 5.9.0)
+            power_assert (0.2.6 < 0.3.0)
+            psych (2.0.17 < 2.1.0)
+        """
+        outdated = {}
+
+        output = self.run_cli("outdated")
+
+        regexp = re.compile(r"(\S+) \((\S+) < (\S+)\)")
+        for package in output.splitlines():
+            match = regexp.match(package)
+            if match:
+                package_id, current_version, latest_version = match.groups()
+                outdated[package_id] = {
+                    "id": package_id,
+                    "name": package_id,
+                    "installed_version": parse_version(current_version),
+                    "latest_version": parse_version(latest_version),
+                }
+
+        return outdated
+
     def search(self, query, extended, exact):
         """Fetch matching packages from ``gem search`` output.
 
@@ -131,24 +163,24 @@ class Gem(PackageManager):
 
         output = self.run_cli("search", query, "--versions", search_arg)
 
-        if output:
-            regexp = re.compile(
-                r"""
-                (?P<package_id>\S+)     # Any string.
-                \                       # A space.
-                \(                      # Start of content in parenthesis.
-                    (?P<version>\S+)    # Version string.
-                    (?:\ \S+)?          # Optional platform value after space.
-                \)
-                """,
-                re.VERBOSE,
-            )
-            for package_id, version in regexp.findall(output):
-                matches[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "latest_version": parse_version(version),
-                }
+        regexp = re.compile(
+            r"""
+            (?P<package_id>\S+)     # Any string.
+            \                       # A space.
+            \(                      # Start of content in parenthesis.
+                (?P<version>\S+)    # Version string.
+                (?:\ \S+)?          # Optional platform value after space.
+            \)
+            """,
+            re.VERBOSE,
+        )
+
+        for package_id, version in regexp.findall(output):
+            matches[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "latest_version": parse_version(version),
+            }
 
         return matches
 
@@ -157,7 +189,7 @@ class Gem(PackageManager):
 
         .. code-block:: shell-session
 
-            ► gem install --user-install markdown
+            ► gem install --user-install --quiet markdown
             Fetching kramdown-2.3.1.gem
             Fetching concurrent-ruby-1.1.9.gem
             (...)
@@ -176,76 +208,50 @@ class Gem(PackageManager):
         """
         super().install(package_id)
         return self.run_cli(
-            "install", "--user-install", self.global_args, package_id, skip_globals=True
+            "install",
+            "--user-install",
+            *self.post_args,
+            package_id,
+            auto_post_args=False,
         )
 
-    @property
-    def outdated(self):
-        """Fetch outdated packages from ``gem outdated`` output.
+    def upgrade_cli(self, package_id=None):
+        """.. code-block:: shell-session.
 
-        Raw CLI output samples:
+            ► gem update --user-install --quiet markdown
+
+        Installs require ``sudo`` on system ruby. I (@tresni) recommend doing something like:
 
         .. code-block:: shell-session
 
-            ► gem outdated
-            did_you_mean (1.0.0 < 1.0.2)
-            io-console (0.4.5 < 0.4.6)
-            json (1.8.3 < 2.0.1)
-            minitest (5.8.3 < 5.9.0)
-            power_assert (0.2.6 < 0.3.0)
-            psych (2.0.17 < 2.1.0)
+            ► sudo dseditgroup -o edit -a -t user wheel
+
+        And then do ``visudo`` to make it so the ``wheel`` group does not require
+        a password. There is a line already there for it, you just need to
+        uncomment it and save.
         """
-        outdated = {}
-
-        output = self.run_cli("outdated")
-
-        if output:
-            regexp = re.compile(r"(\S+) \((\S+) < (\S+)\)")
-            for package in output.splitlines():
-                match = regexp.match(package)
-                if match:
-                    package_id, current_version, latest_version = match.groups()
-                    outdated[package_id] = {
-                        "id": package_id,
-                        "name": package_id,
-                        "installed_version": parse_version(current_version),
-                        "latest_version": parse_version(latest_version),
-                    }
-
-        return outdated
-
-    def upgrade_cli(self, package_id=None):
-        cmd = [self.cli_path, "update", "--user-install", self.global_args]
-        # Installs require `sudo` on system ruby.
-        # I (@tresni) recommend doing something like:
-        #     ► sudo dseditgroup -o edit -a -t user wheel
-        # And then do `visudo` to make it so the `wheel` group does not require
-        # a password. There is a line already there for it, you just need to
-        # uncomment it and save.)
-        # if self.cli_path == '/usr/bin/gem':
-        #     cmd.insert(0, '/usr/bin/sudo')
-        if package_id:
-            cmd.append(package_id)
-        return cmd
+        return self.build_cli(
+            "update",
+            "--user-install",
+            *self.post_args,
+            package_id,
+            auto_post_args=False,
+        )
 
     def cleanup(self):
-        """ Run ``gem cleanup`` CLI.
+        """.. code-block:: shell-session.
 
-        Raw CLI output samples:
-
-        .. code-block:: shell-session
-
-            ► gem cleanup
-            Cleaning up installed gems...
-            Attempting to uninstall test-unit-3.2.9
-            Unable to uninstall test-unit-3.2.9:
-                Gem::FilePermissionError: You don't have write permissions \
-                for the /Library/Ruby/Gems/2.6.0 directory.
-            Attempting to uninstall did_you_mean-1.3.0
-            Unable to uninstall did_you_mean-1.3.0:
-                Gem::FilePermissionError: You don't have write permissions \
-                for the /Library/Ruby/Gems/2.6.0 directory.
-            Clean up complete
+        ► gem cleanup --quiet
+        Cleaning up installed gems...
+        Attempting to uninstall test-unit-3.2.9
+        Unable to uninstall test-unit-3.2.9:
+            Gem::FilePermissionError: You don't have write permissions \
+            for the /Library/Ruby/Gems/2.6.0 directory.
+        Attempting to uninstall did_you_mean-1.3.0
+        Unable to uninstall did_you_mean-1.3.0:
+            Gem::FilePermissionError: You don't have write permissions \
+            for the /Library/Ruby/Gems/2.6.0 directory.
+        Clean up complete
         """
         super().cleanup()
         self.run_cli("cleanup")
