@@ -17,23 +17,23 @@
 
 import re
 
-from click_extra.logging import logger
 from click_extra.platform import MACOS
 
+from .. import logger
 from ..base import PackageManager
 from ..version import parse_version
 
 
 class MAS(PackageManager):
 
+    name = "Mac AppStore"
+
     platforms = frozenset({MACOS})
 
     # 'mas search' output has been fixed in 1.6.1:
     # https://github.com/mas-cli/mas/pull/205
     requirement = "1.6.1"
 
-    name = "Mac AppStore"
-
     version_cli_options = ("version",)
     """
     .. code-block:: shell-session
@@ -58,23 +58,62 @@ class MAS(PackageManager):
 
         output = self.run_cli("list")
 
-        if output:
-            regexp = re.compile(r"(\d+) (.*) \((\S+)\)$")
-            for package in output.splitlines():
-                match = regexp.match(package)
-                if match:
-                    package_id, package_name, installed_version = match.groups()
-                    installed[package_id] = {
-                        "id": package_id,
-                        "name": package_name,
-                        # Normalize unknown version. See:
-                        # https://github.com/mas-cli/mas/commit
-                        # /1859eaedf49f6a1ebefe8c8d71ec653732674341
-                        "installed_version": parse_version(installed_version),
-                    }
+        regexp = re.compile(r"(\d+) (.*) \((\S+)\)$")
+        for package in output.splitlines():
+            match = regexp.match(package)
+            if match:
+                package_id, package_name, installed_version = match.groups()
+                installed[package_id] = {
+                    "id": package_id,
+                    "name": package_name,
+                    # Normalize unknown version. See:
+                    # https://github.com/mas-cli/mas/commit
+                    # /1859eaedf49f6a1ebefe8c8d71ec653732674341
+                    "installed_version": parse_version(installed_version),
+                }
 
         return installed
 
+    @property
+    def outdated(self):
+        """Fetch outdated packages from ``mas outdated`` output.
+
+        Raw CLI output samples:
+
+        .. code-block:: shell-session
+
+            ► mas outdated
+
+        .. todo
+
+            An example of ``mas outdated`` output is missing above.
+        """
+        outdated = {}
+
+        output = self.run_cli("outdated")
+
+        regexp = re.compile(r"(\d+) (.*) \((\S+) -> (\S+)\)$")
+        for package in output.splitlines():
+            match = regexp.match(package)
+            if match:
+                (
+                    package_id,
+                    package_name,
+                    installed_version,
+                    latest_version,
+                ) = match.groups()
+                outdated[package_id] = {
+                    "id": package_id,
+                    "name": package_name,
+                    "latest_version": parse_version(latest_version),
+                    # Normalize unknown version. See:
+                    # https://github.com/mas-cli/mas/commit
+                    # /1859eaedf49f6a1ebefe8c8d71ec653732674341
+                    "installed_version": parse_version(installed_version),
+                }
+
+        return outdated
+
     def search(self, query, extended, exact):
         """Fetch matching packages from ``mas search`` output.
 
@@ -98,32 +137,29 @@ class MAS(PackageManager):
 
         output = self.run_cli("search", query)
 
-        if output:
-            regexp = re.compile(
-                r"""
-                (?P<package_id>\d+)
-                \s+
-                (?P<package_name>.+?)
-                \s+
-                \(
-                    (?P<version>\S+)
-                \)
-                """,
-                re.MULTILINE | re.VERBOSE,
-            )
+        regexp = re.compile(
+            r"""
+            (?P<package_id>\d+)
+            \s+
+            (?P<package_name>.+?)
+            \s+
+            \(
+                (?P<version>\S+)
+            \)
+            """,
+            re.MULTILINE | re.VERBOSE,
+        )
 
-            for package_id, package_name, version in regexp.findall(output):
-
-                # Filters out fuzzy matches, only keep stricly matching
-                # packages.
-                if exact and query not in (package_id, package_name):
-                    continue
-
-                matches[package_id] = {
-                    "id": package_id,
-                    "name": package_name,
-                    "latest_version": parse_version(version),
-                }
+        for package_id, package_name, version in regexp.findall(output):
+            # Filters out fuzzy matches, only keep stricly matching
+            # packages.
+            if exact and query not in (package_id, package_name):
+                continue
+            matches[package_id] = {
+                "id": package_id,
+                "name": package_name,
+                "latest_version": parse_version(version),
+            }
 
         return matches
 
@@ -133,54 +169,12 @@ class MAS(PackageManager):
         .. code-block:: shell-session
 
             ► mas install 945397020
-
         """
         super().install(package_id)
         return self.run_cli("install", package_id)
 
-    @property
-    def outdated(self):
-        """Fetch outdated packages from ``mas outdated`` output.
-
-        Raw CLI output samples:
-
-        .. code-block:: shell-session
-
-            ► mas outdated
-
-        .. todo
-
-            An example of ``mas outdated`` output is missing above.
-        """
-        outdated = {}
-
-        output = self.run_cli("outdated")
-
-        if output:
-            regexp = re.compile(r"(\d+) (.*) \((\S+) -> (\S+)\)$")
-            for package in output.splitlines():
-                match = regexp.match(package)
-                if match:
-                    (
-                        package_id,
-                        package_name,
-                        installed_version,
-                        latest_version,
-                    ) = match.groups()
-                    outdated[package_id] = {
-                        "id": package_id,
-                        "name": package_name,
-                        "latest_version": parse_version(latest_version),
-                        # Normalize unknown version. See:
-                        # https://github.com/mas-cli/mas/commit
-                        # /1859eaedf49f6a1ebefe8c8d71ec653732674341
-                        "installed_version": parse_version(installed_version),
-                    }
-
-        return outdated
-
     def upgrade_cli(self, package_id=None):
-        cmd = [self.cli_path, "upgrade"]
-        if package_id:
-            cmd.append(package_id)
-        return cmd
+        return self.build_cli(
+            "upgrade",
+            package_id,
+        )
