@@ -27,7 +27,7 @@ class Snap(PackageManager):
 
     requirement = "2.0.0"
 
-    global_args = ("--color=never",)
+    pre_args = ("--color=never",)
 
     version_regex = r"snap\s+(?P<version>\S+)"
     """
@@ -48,7 +48,7 @@ class Snap(PackageManager):
         Raw CLI output samples:
 
         .. code-block:: shell-session
-            ► snap list
+            ► snap --color=never list
             Name    Version    Rev   Aufzeichnung   Herausgeber     Hinweise
             core    16-2.44.1  8935  latest/stable  canonical✓      core
             wechat  2.0        7     latest/stable  ubuntu-dawndiy  -
@@ -58,72 +58,17 @@ class Snap(PackageManager):
 
         output = self.run_cli("list")
 
-        if output:
-            for package in output.splitlines()[1:]:
-                package_id = package.split()[0]
-                installed_version = package.split()[1]
-                installed[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "installed_version": parse_version(installed_version),
-                }
+        for package in output.splitlines()[1:]:
+            package_id = package.split()[0]
+            installed_version = package.split()[1]
+            installed[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "installed_version": parse_version(installed_version),
+            }
 
         return installed
 
-    def search(self, query, extended, exact):
-        """Fetch matching packages from ``snap find`` output.
-
-        .. code-block:: shell-session
-            ► snap find doc
-            Name       Version      Herausgeber  Hinweise  Zusammenfassung
-            journey    2.14.3       2appstudio   -         Your private diary.
-            nextcloud  17.0.5snap1  nextcloud✓   -         Nextcloud Server
-            skype      8.58.0.93    skype✓       classic   One Skype for all.
-        """
-        matches = {}
-
-        output = self.run_cli("find", query)
-
-        if output:
-
-            for package in output.splitlines()[1:]:
-
-                package_id = package.split()[0]
-                version = package.split()[1]
-                description = " ".join(map(str, package.split()[4:]))
-
-                # Skip all non-stricly matching package IDs in exact mode.
-                if exact:
-                    if query != package_id:
-                        continue
-
-                else:
-                    # Exclude packages not featuring the search query in their
-                    # ID or name.
-                    if not extended:
-                        query_parts = set(map(str, TokenizedString(query)))
-                        pkg_parts = set(map(str, TokenizedString(package_id)))
-                        if not query_parts.issubset(pkg_parts):
-                            continue
-
-                matches[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "latest_version": parse_version(version),
-                }
-        return matches
-
-    def install(self, package_id):
-        """Install one package.
-
-        .. code-block:: shell-session
-
-            ► snap install standard-notes
-
-        """
-        super().install(package_id)
-        return self.run_cli("install", package_id)
-
     @property
     def outdated(self):
         """Fetch outdated packages from ``snap refresh --list`` output.
@@ -132,7 +77,7 @@ class Snap(PackageManager):
 
         .. code-block:: shell-session
 
-            ► snap refresh --list
+            ► snap --color=never refresh --list
             Name            Version  Rev  Herausgeber     Hinweise
             standard-notes  3.3.5    8    standardnotes✓  -
         """
@@ -140,27 +85,80 @@ class Snap(PackageManager):
 
         output = self.run_cli("refresh", "--list")
 
-        if output and len(output.splitlines()) > 1:
-            for package in output.splitlines()[1:]:
-                package_id = package.split()[0]
-                latest_version = package.split()[1]
-                installed_version = (
-                    self.run_cli("list", package_id).splitlines()[-1].split()[1]
-                )
-                outdated[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "latest_version": parse_version(latest_version),
-                    "installed_version": parse_version(installed_version),
-                }
+        for package in output.splitlines()[1:]:
+            package_id = package.split()[0]
+            latest_version = package.split()[1]
+            installed_version = (
+                self.run_cli("list", package_id).splitlines()[-1].split()[1]
+            )
+            outdated[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "latest_version": parse_version(latest_version),
+                "installed_version": parse_version(installed_version),
+            }
 
         return outdated
 
-    def upgrade_cli(self, package_id=None):
-        """Snap has an auto-update function, but snaps can be updated
-        manually.
+    def search(self, query, extended, exact):
+        """Fetch matching packages from ``snap find`` output.
+
+        .. code-block:: shell-session
+            ► snap --color=never find doc
+            Name       Version      Herausgeber  Hinweise  Zusammenfassung
+            journey    2.14.3       2appstudio   -         Your private diary.
+            nextcloud  17.0.5snap1  nextcloud✓   -         Nextcloud Server
+            skype      8.58.0.93    skype✓       classic   One Skype for all.
         """
-        cmd = [self.cli_path, self.global_args, "refresh"]
-        if package_id:
-            cmd.append(package_id)
-        return cmd
+        matches = {}
+
+        output = self.run_cli("find", query)
+
+        for package in output.splitlines()[1:]:
+
+            package_id = package.split()[0]
+            version = package.split()[1]
+            description = " ".join(map(str, package.split()[4:]))
+
+            # Skip all non-stricly matching package IDs in exact mode.
+            if exact:
+                if query != package_id:
+                    continue
+
+            else:
+                # Exclude packages not featuring the search query in their
+                # ID or name.
+                if not extended:
+                    query_parts = set(map(str, TokenizedString(query)))
+                    pkg_parts = set(map(str, TokenizedString(package_id)))
+                    if not query_parts.issubset(pkg_parts):
+                        continue
+
+            matches[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "latest_version": parse_version(version),
+            }
+        return matches
+
+    def install(self, package_id):
+        """Install one package.
+
+        .. code-block:: shell-session
+
+            ► snap --color=never install standard-notes
+        """
+        super().install(package_id)
+        return self.run_cli("install", package_id)
+
+    def upgrade_cli(self, package_id=None):
+        """Snap has an auto-update function, but snaps can be updated manually.
+
+        .. code-block:: shell-session
+
+            ► snap --color=never refresh standard-notes
+        """
+        return self.build_cli(
+            "refresh",
+            package_id,
+        )
