@@ -15,7 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-import simplejson as json
+import json
+
 from boltons.iterutils import remap
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
@@ -25,12 +26,12 @@ from ..version import TokenizedString, parse_version
 
 class VSCode(PackageManager):
 
+    name = "Visual Studio Code"
+
     platforms = frozenset({LINUX, MACOS, WINDOWS})
 
     requirement = "1.60.0"
 
-    name = "Visual Studio Code"
-
     cli_names = ("code",)
 
     """
@@ -61,14 +62,13 @@ class VSCode(PackageManager):
 
         output = self.run_cli("--list-extensions", "--show-versions")
 
-        if output:
-            for package in output.splitlines():
-                package_id, installed_version = package.split("@")
-                installed[package_id] = {
-                    "id": package_id,
-                    "name": package_id,
-                    "installed_version": parse_version(installed_version),
-                }
+        for package in output.splitlines():
+            package_id, installed_version = package.split("@")
+            installed[package_id] = {
+                "id": package_id,
+                "name": package_id,
+                "installed_version": parse_version(installed_version),
+            }
 
         return installed
 
