@@ -17,14 +17,14 @@
 
 import pytest
 
-from ..pool import ALL_MANAGER_IDS, DEFAULT_MANAGER_IDS
+from ..pool import pool
 from .test_cli import CLISubCommandTests
 
 
 @pytest.fixture
 def subcmd(create_config):
-    """Seed common subcommand tests with a dummy file and content to allow the
-    CLI to not fail on required file input."""
+    """Seed common subcommand tests with a dummy file and content to allow the CLI to
+    not fail on required file input."""
     toml_path = create_config(
         "dummy.toml",
         """
@@ -40,15 +40,13 @@ class TestRestore(CLISubCommandTests):
         toml_path = create_config(
             "all-managers.toml",
             "".join(
-                (
-                    """
+                """
             [{}]
             blah = 123
             """.format(
-                        m
-                    )
-                    for m in ALL_MANAGER_IDS
+                    m
                 )
+                for m in pool.all_manager_ids
             ),
         )
 
@@ -57,20 +55,18 @@ class TestRestore(CLISubCommandTests):
         assert "all-managers.toml" in result.stderr
         self.check_manager_selection(result)
 
-    @pytest.mark.parametrize("mid", DEFAULT_MANAGER_IDS)
+    @pytest.mark.parametrize("mid", pool.default_manager_ids)
     def test_single_manager(self, invoke, create_config, mid):
         toml_path = create_config(
             "all-managers.toml",
             "".join(
-                (
-                    """
+                """
             [{}]
             blah = 123
             """.format(
-                        m
-                    )
-                    for m in ALL_MANAGER_IDS
+                    m
                 )
+                for m in pool.all_manager_ids
             ),
         )
 
