@@ -18,7 +18,7 @@
 import pytest
 from click_extra.tests.conftest import destructive
 
-from ..pool import DEFAULT_MANAGER_IDS
+from ..pool import pool
 from .test_cli import CLISubCommandTests
 
 
@@ -29,22 +29,22 @@ def subcmd():
 
 class TestUpgrade(CLISubCommandTests):
 
-    """All tests here should me marked as destructive unless --dry-run
-    parameter is passed."""
+    """All tests here should me marked as destructive unless --dry-run parameter is
+    passed."""
 
     def test_default_all_managers_dry_run(self, invoke, subcmd):
         result = invoke("--dry-run", subcmd)
         assert result.exit_code == 0
         self.check_manager_selection(result)
 
-    @pytest.mark.parametrize("mid", DEFAULT_MANAGER_IDS)
+    @pytest.mark.parametrize("mid", pool.default_manager_ids)
     def test_single_manager_dry_run(self, invoke, subcmd, mid):
         result = invoke("--manager", mid, "--dry-run", subcmd)
         assert result.exit_code == 0
         self.check_manager_selection(result, {mid})
 
     @destructive
-    @pytest.mark.parametrize("mid", DEFAULT_MANAGER_IDS)
+    @pytest.mark.parametrize("mid", pool.default_manager_ids)
     def test_single_manager(self, invoke, subcmd, mid):
         result = invoke("--manager", mid, subcmd)
         assert result.exit_code == 0
@@ -53,4 +53,5 @@ class TestUpgrade(CLISubCommandTests):
 
 destructive()(TestUpgrade.test_stats)
 destructive()(TestUpgrade.test_default_all_managers)
+destructive()(TestUpgrade.test_manager_shortcuts)
 destructive()(TestUpgrade.test_manager_selection)
